@@ -39,6 +39,7 @@ from rospkg import RosPack, RosStack, ResourceNotFound
 from .core import RosdepInternalError, InvalidData, rd_debug
 from .model import RosdepDatabase
 from .rospkg_loader import RosPkgLoader
+from .dependency_graph import DependencyGraph
 
 from .sources_list import SourcesListLoader
 
@@ -306,17 +307,22 @@ class RosdepLookup(object):
         """
         Resolve all the rosdep dependencies for *resources* using *installer_context*.
 
-        :param resources: list of resources (e.g. packages), ``[str]]``
+        :param resources: list of resources (e.g. packages), ``[str]``
         :param installer_context: :class:`InstallerContext`
         :param implicit: Install implicit (recursive) dependencies of
             resources.  Default ``False``.
         
-        :returns: (resolutions, errors), ``({str: opaque}, {str: ResolutionError})``.  resolutions maps the installer keys
-          to resolved objects.  Resolved objects are opaque but can be passed into the associated installer for processing.
-          errors maps package names to an :exc:`ResolutionError` or :exc:`KeyError` exception.
+        :returns: (resolutions, errors), ``([(str, [str])], {str: ResolutionError})``.  resolutions provides 
+          an ordered list of resolution tuples.  A resolution tuple's first element is the installer 
+          key (e.g.: apt or homebrew) and the second element is a list of opaque resolution values for that 
+          installer. errors maps package names to an :exc:`ResolutionError` or :exc:`KeyError` exception.
+
+        :raises: :exc:`RosdepInternalError` if unexpected error in constructing dependency graph
+        :raises: :exc:`InvalidData` if a cycle occurs in constructing dependency graph
         """
-        resolutions = defaultdict(list)
+        depend_graph = DependencyGraph()
         errors = {}
+        # TODO: resolutions dictionary should be replaced with resolution model instead of mapping (undefined) keys.
         for resource_name in resources:
             try:
                 rosdep_keys = self.get_rosdeps(resource_name, implicit=implicit)
@@ -326,25 +332,36 @@ class RosdepLookup(object):
                     try:
                         installer_key, resolution, dependencies = \
                                        self.resolve(rosdep_key, resource_name, installer_context)
-                        resolutions[installer_key].append(resolution)
+                        depend_graph[rosdep_key]['installer_key'] = installer_key
+                        depend_graph[rosdep_key]['install_keys'] = list(resolution)
+                        depend_graph[rosdep_key]['dependencies'] = list(dependencies)
                         while dependencies:
                             depend_rosdep_key = dependencies.pop()
+                            # prevent infinite loop
+                            if depend_rosdep_key in depend_graph:
+                                continue
                             installer_key, resolution, more_dependencies = \
                                            self.resolve(depend_rosdep_key, resource_name, installer_context)
                             dependencies.extend(more_dependencies)
-                            resolutions[installer_key].append(resolution)
+                            depend_graph[depend_rosdep_key]['installer_key'] = installer_key
+                            depend_graph[depend_rosdep_key]['install_keys'] = list(resolution)
+                            depend_graph[depend_rosdep_key]['dependencies'] = list(more_dependencies)
 
                     except ResolutionError as e:
                         errors[resource_name] = e
             except ResourceNotFound as e:
                 errors[resource_name] = e
 
-        # consolidate resolutions 
-        for installer_key, val in resolutions.items(): #py3k
-            installer = installer_context.get_installer(installer_key)
-            resolutions[installer_key] = installer.unique(*val)
+        try:
+            # TODO: I really don't like AssertionErrors here; this should be modeled as 'CyclicGraphError' 
+            # or something more explicit. No need to continue if this API errors.
+            resolutions_flat = depend_graph.get_ordered_dependency_list()
+        except AssertionError as e:
+            raise InvalidData("cycle in dependency graph detected: %s"%(e))
+        except KeyError as e:
+            raise RosdepInternalError(e)
 
-        return resolutions, errors
+        return resolutions_flat, errors
 
     def resolve(self, rosdep_key, resource_name, installer_context):
         """
