@@ -31,6 +31,7 @@ from __future__ import print_function
 
 import os
 import sys
+import tempfile
 import yaml
 import hashlib
 import urllib2
@@ -392,10 +393,10 @@ def update_sources_list(sources_list_dir=None, sources_cache_dir=None,
     if not os.path.exists(sources_cache_dir):
         os.makedirs(sources_cache_dir)
     cache_index = os.path.join(sources_cache_dir, CACHE_INDEX)
-    with open(cache_index, 'w') as f:
-        f.write("#autogenerated by rosdep, do not edit. use 'rosdep update' instead\n")
-        for source in sources:
-            f.write("yaml %s %s\n"%(source.url, ' '.join(source.tags)))
+    data = "#autogenerated by rosdep, do not edit. use 'rosdep update' instead\n"
+    for source in sources:
+        data += "yaml %s %s\n" % (source.url, ' '.join(source.tags))
+    write_atomic(cache_index, data)
     # mainly for debugging and testing
     return retval
 
@@ -439,10 +440,29 @@ def write_cache_file(source_cache_d, filename_key, rosdep_data):
         os.makedirs(source_cache_d)
     key_hash = compute_filename_hash(filename_key)
     filepath = os.path.join(source_cache_d, key_hash)
-    with open(filepath, 'w') as f:
-        f.write(yaml.safe_dump(rosdep_data))
+    write_atomic(filepath, yaml.safe_dump(rosdep_data))
     return filepath
     
+def write_atomic(filepath, data):
+    # write data to new file
+    fd, filepath_tmp = tempfile.mkstemp(prefix=os.path.basename(filepath) + '.tmp.', dir=os.path.dirname(filepath))
+    with os.fdopen(fd, 'w') as f:
+        f.write(data)
+        f.close()
+    try:
+        # switch file atomically (if supported)
+        os.rename(filepath_tmp, filepath)
+    except OSError:
+        # fall back to non-atomic operation
+        try:
+            os.unlink(filepath)
+        except OSError:
+            pass
+        try:
+            os.rename(filepath_tmp, filepath)
+        except OSError:
+            os.unlink(filepath_tmp)
+
 class SourcesListLoader(RosdepLoader):
     """
     SourcesList loader implements the general RosdepLoader API.  This
